package ch.bouquet.ejad;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import ch.bouquet.ejad.xml.BookType;
import ch.bouquet.ejad.xml.Catalog;

/**
 * <p>Kurs: EADJ03 - 2007
 * <br/>Titel: XML - Uebung 5 (Java Code Generierung mit JAXB)
 * <br/>Beschreibung: Grundlage fuer diese Uebung ist das XML-Schema aus der Uebung 4.
 * <br/>Übung: Java Code Generierung
 * <br/>Lassen Sie sich aus dem vorher generierten Schema (Uebung 4) Java-Klassen generieren. 
 * Schreiben Sie dann einen Testprogramm, welches die Buchdaten in Java einliest (unmarshal) und in
 * einen Baum (JTree) verpackt. Fuer das Lesen der Daten in einen Baum koennen sie das gegebene tree
 * Package benutzen.
 * </p> 
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   marc at bouquet dot ch
 * @version  1.0 - 23.03.2007
 * @see JFrame
 */
public class Schema2jaxb extends JFrame
{
  /**
   * Package name of jaxb classes
   */
  private final static String PACKAGE_NAME = "ch.bouquet.ejad.xml";

  /**
   * Default mutable tree node
   */
  private DefaultMutableTreeNode dmt;

  /**
   * Variable of root element
   */
  private String rootElement;

  /**
   * Default constructor
   * @param args String path of xml-file
   */
  public Schema2jaxb( String args )
  {
    try
    {
      // load the xml file and parsed it.
      Catalog root = loadXml( args );
      if ( root != null )
      {
        this.dmt = new DefaultMutableTreeNode( getRootElement() );
        JTree tree = new JTree( this.dmt );

        // set the notes to the tree
        handleBooks( root.getBook() );
        super.add( new JScrollPane( tree ) );
        TreeNode rootNode = (TreeNode) tree.getModel().getRoot();
        // Traverse tree from root
        expandAll( tree, new TreePath( rootNode ), true );
      }
      else
      {
        addMsg( "Root-Element konnte nicht gelesen werden!" );
      }
    }
    catch ( Exception ex )
    {
      addMsg( ex.getMessage() );
    }
  }

  /**
   * This method read the xml file
   * @param path String args with path to the xml-file
   */
  private Catalog loadXml( String path ) throws Exception
  {
    try
    {
      JAXBContext jc = JAXBContext.newInstance( PACKAGE_NAME );
      Unmarshaller unm = jc.createUnmarshaller();
      Catalog root = (Catalog) unm.unmarshal( new FileInputStream( path ) );
      setRootElement( root.getClass().getName().replaceFirst( PACKAGE_NAME + ".", "" ) );
      return root;
    }
    catch ( JAXBException jex )
    {
      jex.printStackTrace();
      throw new Exception( "Beim Parsen der XML ist ein Fehler aufgetreten! " );
    }
    catch ( FileNotFoundException fex )
    {
      fex.printStackTrace();
      throw new Exception( "XML-Dokument wurde nicht gefunden!" );
    }
  }

  /**
   * This method handle the book elements
   */
  private void handleBooks( List list )
  {
    Iterator iter = list.iterator();
    while ( iter.hasNext() )
    {
      BookType element = (BookType) iter.next();
      StringBuffer buf = new StringBuffer();
      buf.append( element.getTitle() );
      buf.append( " (" );
      buf.append( element.getYear() );
      buf.append( ")" );
      addNotes( buf.toString() );
    }
  }

  /**
   * If expand is true, expands all nodes in the tree.
   * Otherwise, collapses all nodes in the tree.
   * @param tree
   * @param parent
   * @param expand
   */
  private void expandAll( JTree tree, TreePath parent, boolean expand )
  {
    // Traverse children
    TreeNode node = (TreeNode) parent.getLastPathComponent();
    if ( node.getChildCount() >= 0 )
    {
      for ( Enumeration e = node.children(); e.hasMoreElements(); )
      {
        TreeNode n = (TreeNode) e.nextElement();
        TreePath path = parent.pathByAddingChild( n );
        expandAll( tree, path, expand );
      }
    }

    // Expansion or collapse must be done bottom-up
    if ( expand )
    {
      tree.expandPath( parent );
    }
    else
    {
      tree.collapsePath( parent );
    }
  }

  /**
   * Insert new notes in the tree
   * @param value note name
   */
  private void addNotes( String value )
  {
    this.dmt.add( new DefaultMutableTreeNode( value ) );
  }

  /**
   * This method set the error message in the windows
   * @param msg String error messages
   */
  private void addMsg( String msg )
  {
    super.add( new JLabel( msg ) );
  }

  /**
   * getter method
   * @return String name of root element
   */
  public String getRootElement()
  {
    return rootElement;
  }

  /**
   * setter method
   * @param value String name of root element
   */
  public void setRootElement( String value )
  {
    this.rootElement = value;
  }

  /**
   * Main-method
   * @param args [0] = path to the xml file
   */
  public static void main( String[] args )
  {
    Schema2jaxb jb = new Schema2jaxb( args[0] );
    jb.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
    jb.setBounds( 100, 100, 350, 180 );
    jb.setTitle( "EJAD03- XML - Uebung 4" );
    jb.setVisible( true );
  }
}