<?xml version="1.0" encoding="UTF-8"?>
<!--
    ==================================================================================
    Kurs:                              EADJ03 - 2007
    
    Titel:                               XML - Uebung 6 (XSL-Stylesheet)
    
    Beschreibung:             Schreiben Sie ein Stylesheet, welches diese Ausgabe (Lösung -URL einfügen) 
                                           generiert.
    
                                           Die verschiedenen Tabellen sind: 
                                           - nur die Buecher mit mehr als 600 Seiten, sortiert zuerst nach Anzahl Seiten, dann
                                             nach Buchtitel. 
                                           - nur die Buecher, die mehr als einen Autor haben. 
                                           - Buchtitel sind (abhängig vom Erscheinungsdatum) verschieden gefaerbt. 
                                           - Buecher aufgelistet nach Autor. 
    
                                           Um das exakt gleiche Aussehen zu erreichen, koennen Sie das css Stylesheet unter 
                                           http://www.sws.bfh.ch/~amrhein/XSL/EADPraktikum/style.css einbinden. 
    
    Rahmenprogramm:   Als Basis dient das folgende XML-Dokument:
                                           http://www.sws.bfh.ch/~amrhein/XML/catalog.xml
    
    Autor:                             marc at bouquet dot ch
    
    Version:                        1.0 - 23.03.2007
    ==================================================================================
-->

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<xsl:output encoding="UTF-8" method="xml"/>

	<xsl:variable name="author-ref" select="catalog/authors"/>
	<xsl:variable name="publisher-ref" select="catalog/publishers"/>

	<xsl:key name="authorKey" match="author" use="@id"/>
	<xsl:key name="publisherKey" match="publisher" use="@pID"/>
	<xsl:key name="list" match="authorRef" use="@aRef"/>

	<!-- ROOT-ELEMENT -->
	<xsl:template match="catalog">
		<html xmlns="http://www.w3.org/1999/xhtml" lang="de">
			<head>
				<title>XML - Uebung 6 (XSL-Stylesheet) - EADJ03 - 2007</title>
				<link rel="stylesheet" title="Default" href="style.css" type="text/css"/>
			</head>
			<body>
				<a name="anchor_0"/>
				<xsl:call-template name="instruction"/>
				<!--  all books  -->
				<h1>
					<font color="green">Alle Buecher</font>
				</h1>
				<table>
					<xsl:call-template name="table-head"/>
					<xsl:apply-templates select="book">
						<xsl:sort select="title" order="ascending"/>
					</xsl:apply-templates>
				</table>
				<a href="#anchor_0">[top]</a>

				<!-- only Books with more than 600 pages, sorted by number of pages and book title -->
				<a name="anchor_1"/>
				<h1>
					<font color="green">Buecher mit mehr als 600 Seiten</font>
				</h1>
				<h2>Sortiert nach Anzahl Seiten und Buchtitel</h2>
				<table>
					<xsl:call-template name="table-head"/>
					<xsl:apply-templates select="book" mode="tab_1">
						<xsl:sort select="pages" order="ascending" data-type="number"/>
						<xsl:sort select="title" order="ascending"/>
					</xsl:apply-templates>
				</table>
				<a href="#anchor_0">[top]</a>

				<!-- books with more than one Author -->
				<a name="anchor_2"/>
				<h1>
					<font color="green">Buecher mit mehr als einem Autor</font>
				</h1>
				<table>
					<xsl:call-template name="table-head"/>
					<xsl:apply-templates select="book" mode="tab_2">
						<xsl:sort select="pages" order="ascending" data-type="number"/>
						<xsl:sort select="title" order="ascending"/>
					</xsl:apply-templates>
				</table>
				<a href="#anchor_0">[top]</a>

				<!-- title of new books are red or green, title of older books are blue -->
				<a name="anchor_3"/>
				<h1>
					<font color="green">Buecher sind verschieden gefaerbt</font>
				</h1>
				<h2>abhaengig vom Erscheinungsdatum</h2>
				<table>
					<xsl:call-template name="table-head"/>
					<xsl:apply-templates select="book" mode="tab_3"/>
				</table>
				<a href="#anchor_0">[top]</a>
				<!-- books per Author -->
				<a name="anchor_4"/>
				<h1>
					<font color="green">Buecher aufgelistet nach Autor</font>
				</h1>
				<table>
					<tr>
						<th width="16%">Author</th>
						<th width="84%">Book(s)</th>
					</tr>
					<xsl:for-each
						select="book/authorRef[generate-id(.)=generate-id(key('list', @aRef))]/@aRef">
						<xsl:sort select="../author/firstName" order="ascending"/>
						<tr>
							<xsl:call-template name="css-tr-row"/>
							<td valign="top">
								<strong>
									<xsl:apply-templates select=".."/>
								</strong>
							</td>
							<td valign="top">
								<table>
									<tr>
										<th width="33%">Title</th>
										<th width="33%">Publisher</th>
										<th width="33%">ISBN</th>
									</tr>
									<xsl:for-each select="key('list', .)">
										<xsl:sort select="../title" order="ascending"/>
										<tr>
											<xsl:call-template name="css-tr-row"/>
											<xsl:apply-templates select="../." mode="tab_4"/>
										</tr>
									</xsl:for-each>
								</table>
							</td>
						</tr>
					</xsl:for-each>
				</table>
				<a href="#anchor_0">[top]</a>
			</body>
		</html>
	</xsl:template>

	<!-- T E M P L A T E S  -->
	<!-- all books -->
	<xsl:template match="book">
		<xsl:call-template name="book"/>
	</xsl:template>

	<!-- only Books with more than 600 pages, sorted by number of pages and book title -->
	<xsl:template match="book" mode="tab_1">
		<xsl:if test="pages > 600">
			<xsl:call-template name="book"/>
		</xsl:if>
	</xsl:template>

	<!-- books with more than one Author -->
	<xsl:template match="book" mode="tab_2">
		<xsl:variable name="countAuthors" select="count(authorRef/@aRef)"/>
		<xsl:if test="$countAuthors > 1">
			<xsl:call-template name="book"/>
		</xsl:if>
	</xsl:template>

	<!-- title of new books are red or green, title of older books are blue  -->
	<xsl:template match="book" mode="tab_3">
		<tr>
			<xsl:call-template name="css-tr-row"/>
			<td>
				<xsl:choose>
					<xsl:when test="date/year &lt; 2004">
						<font color="blue">
							<xsl:call-template name="book-title"/>
						</font>
					</xsl:when>
					<xsl:when test="date/year = 2004">
						<font color="green">
							<xsl:call-template name="book-title"/>
						</font>
					</xsl:when>
					<xsl:when test="date/year &gt; 2004">
						<font color="red">
							<xsl:call-template name="book-title"/>
						</font>
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="book-title"/>
					</xsl:otherwise>
				</xsl:choose>
			</td>
			<td>
				<xsl:apply-templates select="authorRef "/>
			</td>
			<td>
				<xsl:call-template name="book-publisher"/>
			</td>
			<td>
				<xsl:value-of select="ISBN"/>
			</td>
			<td>
				<xsl:value-of select="pages"/>
			</td>
			<td>
				<xsl:apply-templates select="date"/>
			</td>
			<td>
				<xsl:apply-templates select="price"/>
			</td>
			<td>
				<xsl:value-of select="quantity"/>
				<xsl:if test="not(quantity)">0</xsl:if>
			</td>
		</tr>
	</xsl:template>
	
	<!-- books per Author -->
	<xsl:template match="book" mode="tab_4">
		<td>
			<xsl:call-template name="book-title"/>
		</td>
		<td>
			<xsl:call-template name="book-publisher"/>
		</td>
		<td>
			<xsl:value-of select="ISBN"/>
		</td>
	</xsl:template>
	
	<!-- divers templates -->
	<xsl:template match="authorRef">
		<xsl:if test="position() != 1">
			<br/>
		</xsl:if>
		<xsl:variable name="vRef" select="@aRef"/>
		<xsl:for-each select="$author-ref">
			<xsl:apply-templates select="key('authorKey', $vRef)"/>
		</xsl:for-each>
	</xsl:template>
	
	<xsl:template match="author">
		<xsl:value-of select="firstName"/>
		<xsl:text>&#x0020;</xsl:text>
		<xsl:value-of select="middleName"/>
		<xsl:text>&#x0020;</xsl:text>
		<xsl:value-of select="lastName"/>
	</xsl:template>
	
	<xsl:template match="publisher">
		<xsl:element name="a">
			<xsl:attribute name="href">
				<xsl:text>http://</xsl:text>
				<xsl:value-of select="@url"/>
			</xsl:attribute>
			<xsl:attribute name="target">
				<xsl:text>_blank</xsl:text>
			</xsl:attribute>
			<xsl:value-of select="node()"/>
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="date">
		<xsl:value-of select="month"/>
		<xsl:text>, </xsl:text>
		<xsl:value-of select="year"/>
	</xsl:template>
	
	<xsl:template match="price">
		<xsl:value-of select="@currency"/>
		<xsl:text> </xsl:text>
		<xsl:value-of select="node()"/>
		<xsl:if test="position() != last()">
			<br/>
		</xsl:if>
	</xsl:template>
	
	<xsl:template name="instruction">
		<h1>XML - Uebung 6 (XSL-Stylesheet)</h1>
		<table>
			<tr>
				<th colspan="2">Aufgabenstellung</th>
			</tr>
			<tr class="row2">
				<td>Kurs:</td>
				<td>EADJ03 - 2007</td>
			</tr>
			<tr valign="top" class="row1">
				<td>Titel:</td>
				<td>XML - Uebung 6 (XSL-Stylesheet)</td>
			</tr>
			<tr valign="top" class="row2">
				<td>Beschreibung:</td>
				<td> Schreiben Sie ein Stylesheet, welches diese <a
						href="http://www.sws.bfh.ch/~amrhein/XSL/EADPraktikum/EADLoes.html"
						target="_blank">Ausgabe</a> generiert. <br/>
					<br/> Die verschiedenen Tabellen sind:<br/> - <a href="#anchor_1">nur die
						Buecher mit mehr als 600 Seiten, sortiert zuerst nach Anzahl Seiten, dann
						nach Buchtitel</a>.<br/> - <a href="#anchor_2">nur die Buecher, die mehr als
						einen Autor haben</a>.<br/> - <a href="#anchor_3">Buchtitel sind (abhaengig
						vom Erscheinungsdatum) verschieden gefaerbt</a>.<br/> - <a href="#anchor_4"
						>Buecher aufgelistet nach Autor</a>. <br/><br/> Um das exakt gleiche
					Aussehen zu erreichen, koennen Sie das css Stylesheet <a
						href="http://www.sws.bfh.ch/~amrhein/XSL/EADPraktikum/style.css"
						target="_blank">einbinden</a>.</td>
			</tr>
			<tr valign="top" class="row1">
				<td>Rahmenprogramm:</td>
				<td>Als Basis dient das folgende XML-Dokument: <a
						href="http://www.sws.bfh.ch/~amrhein/XML/catalog.xml" target="_blank"
						>catalog.xml</a>
				</td>
			</tr>
			<tr valign="top" class="row2">
				<td>Autor:</td>
				<td>
					marc at bouquet dot ch - <a href="http://bouquet.ch" target="_blank">bouquet dot ch</a>
				</td>
			</tr>
			<tr valign="top" class="row1">
				<td>Version:</td>
				<td>1.0 - 23.03.2007</td>
			</tr>
		</table>
	</xsl:template>
	
	<xsl:template name="table-head">
		<tr>
			<th width="16%">Title</th>
			<th width="12%">Author(s)</th>
			<th width="12%">Publisher</th>
			<th width="12%">ISBN</th>
			<th width="12%">Pages</th>
			<th width="12%">Published</th>
			<th width="12%">Price</th>
			<th width="12%">Quantity</th>
		</tr>
	</xsl:template>
	
	<xsl:template name="css-tr-row">
		<xsl:choose>
			<xsl:when test="position() mod 2 = 0">
				<xsl:attribute name="class">row2</xsl:attribute>
			</xsl:when>
			<xsl:otherwise>
				<xsl:attribute name="class">row1</xsl:attribute>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="book">
		<tr>
			<xsl:call-template name="css-tr-row"/>
			<td>
				<xsl:call-template name="book-title"/>
			</td>
			<td>
				<xsl:apply-templates select="authorRef"/>
			</td>
			<td>
				<xsl:call-template name="book-publisher"/>
			</td>
			<td>
				<xsl:value-of select="ISBN"/>
			</td>
			<td>
				<xsl:value-of select="pages"/>
			</td>
			<td>
				<xsl:apply-templates select="date"/>
			</td>
			<td>
				<xsl:apply-templates select="price"/>
			</td>
			<td>
				<xsl:value-of select="quantity"/>
				<xsl:if test="not(quantity)">0</xsl:if>
			</td>
		</tr>
	</xsl:template>
	
	<xsl:template name="book-title">
		<strong>
			<xsl:value-of select="title"/>
		</strong>
		<xsl:if test="subtitle">
			<br/>
		</xsl:if>
		<xsl:value-of select="subtitle"/>
		<xsl:if test="edition">
			<br/>
		</xsl:if>
		<span class="edition">
			<xsl:value-of select="edition"/>
		</span>
	</xsl:template>
	
	<xsl:template name="book-publisher">
		<xsl:variable name="vPubRef" select="@pubRef"/>
		<xsl:for-each select="$publisher-ref">
			<xsl:apply-templates select="key('publisherKey', $vPubRef)"/>
		</xsl:for-each>
	</xsl:template>
</xsl:stylesheet>